@@ -52,29 +52,31 @@ module System.Plugins.Env (
 
 import System.Plugins.LoadTypes (Module)
 import System.Plugins.PackageAPI  {- everything -}
-#if CABAL == 1 || __GLASGOW_HASKELL__ >= 604
-import System.Plugins.ParsePkgConfCabal( parsePkgConf )
-#else
-import System.Plugins.ParsePkgConfLite ( parsePkgConf )
-#endif
-import System.Plugins.Consts           ( ghcLibraryPath, sysPkgConf, sysPkgSuffix )
+import System.Plugins.Consts           ( sysPkgConf, sysPkgSuffix )
 
 import Data.IORef               ( writeIORef, readIORef, newIORef, IORef() )
 import Data.Maybe               ( isJust, isNothing, fromMaybe )
-import Data.List                ( isPrefixOf, nub )
+import Data.List                ( isInfixOf, nub )
 
 import System.IO.Unsafe         ( unsafePerformIO )
+import System.IO                ( hGetContents )
 import System.Directory         ( doesFileExist )
+import System.Process           ( waitForProcess, runInteractiveCommand )
 #if defined(CYGWIN) || defined(__MINGW32__)
 import Prelude hiding ( catch, ioError )
-import System.Environment       ( getEnv )
 import System.IO.Error          ( catch, ioError, isDoesNotExistError )
 #endif
 
 import Control.Concurrent.MVar  ( MVar(), newMVar, withMVar )
 
-import Distribution.Package
-import Text.ParserCombinators.ReadP
+import Distribution.InstalledPackageInfo
+-- import Distribution.Package hiding (packageName, PackageName(..))
+import Distribution.Simple.Compiler
+import Distribution.Simple.GHC
+import Distribution.Simple.PackageIndex
+import Distribution.Simple.Program
+import Distribution.Text
+import Distribution.Verbosity
 
 import qualified Data.Map as M
 import qualified Data.Set as S
@@ -146,6 +148,7 @@ type Env = (MVar (),
             IORef StaticPkgEnv,
             IORef MergeEnv)
 
+
 --
 -- our environment, contains a set of loaded objects, and a map of known
 -- packages and their informations. Initially all we know is the default
@@ -284,9 +287,9 @@ addPkgConf f = do
 union :: PkgEnvs -> [PackageConfig] -> PkgEnvs
 union ls ps' = 
         let fm = emptyFM -- new FM for this package.conf
-        in foldr (\p fm' -> if packageName_ p == "base" -- ghc doesn't supply a version with 'base'
-                                                        -- for some reason.
-                            then addToFM (addToFM fm' (packageName_ p) p) (packageName p) p
+        in foldr (\p fm' -> if (display $ package p) == "base" -- ghc doesn't supply a version with 'base'
+                                                       -- for some reason.
+                            then addToFM (addToFM fm' (display $ package p) p) (packageName p) p
                             else addToFM fm' (packageName p) p) fm ps' : ls
 
 -- 
@@ -297,29 +300,20 @@ union ls ps' =
 --
 
 grabDefaultPkgConf :: IO PkgEnvs
+
 grabDefaultPkgConf = do
-        pkgs <- readPackageConf $ ghcLibraryPath </> sysPkgConf
-        return $ union [] pkgs
+        pkg_confs <- get_ghc_configs
+        packages <- mapM readPackageConf pkg_confs
+        return $ foldl union [] packages
 
 --
 -- parse a source file, expanding any $libdir we see.
 --
 readPackageConf :: FilePath -> IO [PackageConfig]
 readPackageConf f = do
-        s <- readFile f
-        let p = parsePkgConf s
-        return $! map expand_libdir p
-
-  where
-      expand_libdir :: PackageConfig -> PackageConfig
-      expand_libdir pk =
-        let pk'   = updImportDirs  (\idirs -> map expand idirs) pk
-            pk''  = updLibraryDirs (\ldirs -> map expand ldirs) pk'
-        in  pk''
-
-      expand :: FilePath -> FilePath
-      expand s | "$libdir" `isPrefixOf` s = ghcLibraryPath ++ drop 7 s
-      expand s = s
+    pc <- configureAllKnownPrograms silent defaultProgramConfiguration
+    pkgIndex <- getInstalledPackages silent (SpecificPackageDB f) pc
+    return $ allPackages pkgIndex
 
 -- -----------------------------------------------------------
 -- Static package management stuff. A static package is linked with the base
@@ -329,10 +323,7 @@ addStaticPkg :: PackageName -> IO ()
 addStaticPkg pkg = modifyStaticPkgEnv env $ \set -> return $ S.insert pkg set
 
 isStaticPkg :: PackageName -> IO Bool
-isStaticPkg pkg
-    = case readP_to_S parsePackageName pkg of
-        ((pkgName,_):_) -> withStaticPkgEnv env $ \set -> return $ S.member pkgName set
-        [] -> return False
+isStaticPkg pkg = withStaticPkgEnv env $ \set -> return $ S.member pkg set
 
 --
 -- Package path, given a package name, look it up in the environment and
@@ -402,21 +393,21 @@ lookupPkg' p = withPkgEnvs env $ \fms -> go fms p
         go (fm:fms) q = case lookupFM fm q of
             Nothing -> go fms q     -- look in other pkgs
 
-            Just package -> do
-                let    hslibs  = hsLibraries package
-                       extras' = extraLibraries package
+            Just pkg -> do
+                let    hslibs  = hsLibraries pkg
+                       extras' = extraLibraries pkg
                        cbits   = filter (\e -> reverse (take (length "_cbits") (reverse e)) == "_cbits") extras'
                        extras  = filter (flip notElem cbits) extras'
-                       ldopts  = ldOptions package
-                       deppkgs = packageDeps package
+                       ldopts  = ldOptions pkg
+                       deppkgs = packageDeps pkg
                 ldInput <- mapM classifyLdInput ldopts
                 let ldOptsLibs  = [ path | Just (DLL path) <- ldInput ]
                     ldOptsPaths = [ path | Just (DLLPath path) <- ldInput ]
                     dlls        = map mkSOName (extras ++ ldOptsLibs)
 #if defined(CYGWIN) || defined(__MINGW32__)
-                    libdirs = fix_topdir (libraryDirs package) ++ ldOptsPaths
+                    libdirs = fix_topdir (libraryDirs pkg) ++ ldOptsPaths
 #else
-                    libdirs = libraryDirs package ++ ldOptsPaths
+                    libdirs = libraryDirs pkg ++ ldOptsPaths
 #endif
 		-- If we're loading dynamic libs we need the cbits to appear before the
 		-- real packages.
@@ -451,10 +442,10 @@ lookupPkg' p = withPkgEnvs env $ \fms -> go fms p
 	replace_topdir (x:xs)       = x : replace_topdir xs
 #endif
         -- a list elimination form for the Maybe type
-        filterRight :: [Either left right] -> [right]
-        filterRight []           = []
-        filterRight (Right x:xs) = x:filterRight xs
-        filterRight (Left _:xs)  =   filterRight xs
+        --filterRight :: [Either left right] -> [right]
+        --filterRight []           = []
+        --filterRight (Right x:xs) = x:filterRight xs
+        --filterRight (Left _:xs)  =   filterRight xs
 
         --
         -- Check that a path to a library actually reaches a library
@@ -514,3 +505,26 @@ addMerge a b z = modifyMerged env $ \fm -> return $ addToFM fm (a,b) z
 (</>) :: FilePath -> FilePath -> FilePath
 [] </> b = b
 a  </> b = a ++ "/" ++ b
+
+-------------------------------------------------------------------------
+--
+-- 'run_cmd' executes command and returns it's standard output
+-- as 'String'
+
+run_cmd :: String -> IO String
+run_cmd cmd = do (_hI, hO, _hE, hProcess) <- runInteractiveCommand cmd
+                 output <- hGetContents hO
+                 _exitCode <- waitForProcess hProcess
+                 return output
+--
+-- 'get_ghc_configs' returns list of strings of packages.conf files in system
+
+get_ghc_configs :: IO [String]
+get_ghc_configs = do ghc_out <- run_cmd "ghc-pkg list"
+                     let configs = map (reverse.strip_trash.reverse) $
+                                   filter (isInfixOf sysPkgConf) $ lines ghc_out
+                     return configs
+                         -- | strip ":\r?" from string head
+                         where strip_trash [] = []
+                               strip_trash xs@(x:xs') | x `elem` ":\r" = strip_trash xs'
+                                                      | otherwise      = xs
