@@ -69,8 +69,7 @@ import System.Plugins.LoadTypes
 -- import Language.Hi.Parser
 import BinIface
 import HscTypes
-import Module (moduleName, moduleNameString)
-import PackageConfig (packageIdString)
+import Module (moduleName, moduleNameString, packageIdString)
 import HscMain (newHscEnv)
 import TcRnMonad (initTcRnIf)
 
@@ -97,7 +96,7 @@ readBinIface' :: FilePath -> IO ModIface
 readBinIface' hi_path = do
     -- kludgy as hell
     e <- newHscEnv undefined
-    initTcRnIf 'r' e undefined undefined (readBinIface hi_path)
+    initTcRnIf 'r' e undefined undefined (readBinIface IgnoreHiWay QuietBinIFaceReading hi_path)
 
 -- TODO need a loadPackage p package.conf :: IO () primitive
 
@@ -661,7 +660,7 @@ loadDepends obj incpaths = do
                 -- now, try to generate a path to the actual .o file
                 -- fix up hierachical names
                 let mods_ = map (\s -> (s, map (\c -> 
-                        if c == '.' then '/' else c) $ decode s)) ds'
+                        if c == '.' then '/' else c) $ s)) ds'
 
                 -- construct a list of possible dependent modules to load
                 let mods = concatMap (\p -> 
@@ -691,7 +690,7 @@ loadDepends obj incpaths = do
 #if DEBUG
                 when (not (null ps')) $ putStrLn "done" 
                 putStr "Loading object" 
-                mapM_ (\(m,_) -> putStr (" "++(decode m)) >> hFlush stdout) mods''
+                mapM_ (\(m,_) -> putStr (" "++ m) >> hFlush stdout) mods''
 #endif
                 moduleDeps <- mapM (\(hi,m) -> loadObject m (Object hi)) mods''
                 return (hiface,moduleDeps)
